import apkutils2
from adbutils import AdbDevice, RunningAppInfo
from titandevice._titian_exception import *


class _AndroidPackageManager:
    __adb_device: AdbDevice

    def __init__(self, adb_device):
        self.__adb_device = adb_device

    def get_installed_packages(self) -> list[str]:
        return self.__adb_device.list_packages()

    def get_package_info(self, package_name):
        if not self.is_package_installed(package_name):
            raise PackageNoFoundException(self.__adb_device.serial, package_name)
        return self.__adb_device.app_info(package_name)

    def is_package_installed(self, package_name: str) -> bool:
        return package_name in self.get_installed_packages()

    def uninstall_package(self, package_name) -> bool:
        self.__adb_device.uninstall(package_name)
        return not self.is_package_installed(package_name)

    def install_package(self, local_package_path: str, uninstall=True, no_launch=True):
        apk = apkutils2.APK(local_package_path)
        package_name = apk.manifest.package_name

        self.__adb_device.install(
            local_package_path, uninstall=uninstall, nolaunch=no_launch, silent=True
        )
        return self.is_package_installed(package_name)

    def get_current_app(self) -> RunningAppInfo:
        return self.__adb_device.app_current()

    def start(self, package_name):
        self.__adb_device.app_start(package_name)
        return self.get_current_app().package == package_name

    def stop(self, package_name):
        self.__adb_device.app_stop(package_name)
        return self.get_current_app().package != package_name



from adbutils import adb

from titandevice._titian_exception import DeviceMustBeRootedException
from titandevice.android._package_manager import _AndroidPackageManager


class _AndroidDevice:

    def __init__(self, device_serial: str):
        self.__adb_device = adb.device(device_serial)
        self.device_serial = device_serial
        self.device_ip = self.__adb_device.wlan_ip()
        self.device_is_root = self.__is_root()
        self.device_device = self.__adb_device.prop.device
        self.device_model = self.__adb_device.prop.model
        self.device_product = self.__adb_device.prop.name
        self.__package_manager = _AndroidPackageManager(self.__adb_device)

    def get_package_manager(self) -> _AndroidPackageManager:
        return self.__package_manager

    def get_frida_manager(
            self, frida_server_file_name_prefix, frida_server_install_path
    ):
        if not self.device_is_root:
            raise DeviceMustBeRootedException(self.device_serial)
        from titandevice.android._frida_manager import _AndroidFridaManager
        return _AndroidFridaManager(
            self, frida_server_file_name_prefix, frida_server_install_path
        )

    def __is_root(self):
        return self.file_exists("/system/xbin/su") or self.file_exists(
            "/system/bin/su"
        )

    def file_exists(self, path: str) -> bool:
        return 'No such file or directory' not in self.__adb_device.shell(
            f"ls {path}"
        ).strip()

    def remove_file(self, path: str):
        self.shell(f"rm {path}")

    def get_pid(self, target: str) -> int:
        result = self.shell(f"ps -A | grep {target}")
        if result == "":
            return -1
        return int(result.split()[1])

    def get_all_files(self, path: str) -> list[str]:
        return self.shell(f"ls {path}").strip().split()

    def shell(self, cmd: str) -> str:
        if self.device_is_root:
            return self.__adb_device.shell(f"su -c {cmd}")
        return self.__adb_device.shell(cmd)

    def kill_process(self, pid: int):
        self.shell(f"kill -9 {pid}")

    #
    # def __dict__(self):
    #     return {
    #         'serial': self.device_serial,
    #         'product': self._adb_device.prop.name,
    #         'model': self._adb_device.prop.device_model,
    #         'device': self._adb_device.prop.device_device,
    #         'is_root': self.device_is_root,
    #         "ip": self.ip
    #     }
    #
    # def reboot(self):
    #     self._adb_device.reboot()
    #
    # def wlan_ip(self):
    #     return self._adb_device.wlan_ip()
    #
    # def push(self, local_path: str, remote_path: str):
    #     self._adb_device.push(local_path, remote_path)
    def dict(self):
        return {
            'serial': self.device_serial,
            'product': self.device_product,
            'model': self.device_model,
            'device': self.device_device,
            'is_root': self.device_is_root,
            "ip": self.device_ip
        }

    def push(self, source, destination):

        if self.file_exists(destination):
            self.remove_file(destination)
        self.__adb_device.push(source, destination)

    def rename(self, old_name, new_name):
        self.shell(f"mv {old_name} {new_name}")



import click

from titandevice import PackageNoFoundException
from titandevice.android._frida_manager import _AndroidFridaManager
from titandevice.android.device_manager import AndroidDeviceManager

package_name = 'com.sina.weibo'
device_serial = '19301FDF6006EP'
frida_version = '16.2.1'
input_frida_server_paths = [
    '/home/mark/workspace/tools/frida-server/frida-server-16.2.1-android-arm64',
    '/home/mark/workspace/tools/frida-server/frida-server-16.1.11-android-arm64',
    '/home/mark/workspace/tools/frida-server/frida-server-16.2.0-android-arm64'
]
if __name__ == '__main__':
    try:
        device_manager = AndroidDeviceManager()
        # devices = device_manager.get_all_devices()
        # device_serial = questionary.select(
        #     message='请选择一个设备:',
        #     choices=[device.device_serial for device in devices]
        # ).ask()
        device = device_manager.get_device(device_serial)
        click.echo(
            f'您选择的设备是: {device.dict()}'
        )
        package_manager = device.get_package_manager()
        # installed_packages = package_manager.get_installed_packages()
        # click.echo('已安装的包:')
        # for package in installed_packages:
        #     click.echo(f'{package}')
        # package_name = 'com.almatar'
        package_info = package_manager.get_package_info(package_name)
        click.echo(f'包信息: {package_info}')
        # is_ok = package_manager.uninstall_package(package_name)
        # click.echo(f'卸载结果: {is_ok}')
        # is_ok = package_manager.start(package_name)
        # click.echo(f'启动结果: {is_ok}')
        # is_ok = package_manager.stop(package_name)
        # click.echo(f'停止结果: {is_ok}')
        frida_manager: _AndroidFridaManager = device.get_frida_manager(
            "frida-server", "/data/local/tmp/"
        )
        frida_devices = frida_manager.get_all_frida()
        click.echo('Frida列表:')
        #     is_ok = frida.start()
        #     click.echo(f'启动结果: {is_ok}')
        #     is_ok = frida.stop()
        #     click.echo(f'停止结果: {is_ok}')
        # for input_frida_server_path in input_frida_server_paths:
        #     is_ok = frida_manager.install_frida(input_frida_server_path)
        #     click.echo(f'安装结果: {is_ok}')
    except PackageNoFoundException as e:
        click.echo(f'包未找到: {e} 开始安装...')
        is_ok = AndroidDeviceManager().get_device(
            '19301FDF6006EP'
        ).get_package_manager().install_package(
            '/home/mark/workspace/stability_laboratory/weibo/apks/14.4.1.apk'
        )
        click.echo(f'安装结果: {is_ok}')



class TitanException(Exception):
    def __init__(self, message: str):
        super().__init__(message)


class AnyDeviceNoFoundException(TitanException):
    def __init__(self):
        super().__init__("Any device not found.")


class DeviceNoFoundException(TitanException):
    def __init__(self, device_serial: str):
        super().__init__(f"Device with serial {device_serial} not found.")


class PackageNoFoundException(TitanException):
    def __init__(self, device_serial: str, package_name: str):
        super().__init__(f"Package {package_name} not found on device {device_serial}.")


class DeviceMustBeRootedException(TitanException):
    def __init__(self, device_serial: str):
        super().__init__(f"Device with serial {device_serial} must be rooted.")


class AnyFridaNotInstalledException(TitanException):
    def __init__(self):
        super().__init__("Any device frida not installed.")


class FridaNotInstalledException(TitanException):
    def __init__(self, device_serial: str, frida_server_path: str):
        super().__init__(
            f"Frida not installed on device {device_serial} with path {frida_server_path}."
        )



import os.path
import subprocess

from adbutils import adb_path

from titandevice import FridaNotInstalledException
from titandevice.utils import get_file_name_from_path


class _AndroidFrida:

    @staticmethod
    def __check_is_installed(func):

        def wrapper(self, *args, **kwargs):
            if not self.__android_device.file_exists(self.__frida_server_full_path):
                raise FridaNotInstalledException(
                    self.__android_device.device_serial, self.__frida_server_full_path
                )
            return func(self, *args, **kwargs)

        return wrapper

    def __init__(self, android_device, frida_server_executable_file_path):
        from titandevice.android._device import _AndroidDevice
        self.__android_device: _AndroidDevice = android_device
        self.__frida_server_full_path = frida_server_executable_file_path
        self.__frida_server_path = os.path.abspath(frida_server_executable_file_path)
        self.__frida_server_name = get_file_name_from_path(
            frida_server_executable_file_path
        )
        self.__version = self.__get_version()
        self.__is_running = self.__check_if_running()

    @property
    def is_running(self):
        return self.__is_running

    @property
    def version(self):
        return self.__version

    @property
    def frida_server_name(self):
        return self.__frida_server_name

    @property
    def frida_server_path(self):
        return self.__frida_server_path

    @property
    def frida_server_full_path(self):
        return self.__frida_server_full_path

    def __check_if_running(self):
        return self.__android_device.get_pid(self.__frida_server_name) != -1

    @__check_is_installed
    def __get_version(self):
        return self.__android_device.shell(
            f"su -c {self.__frida_server_full_path} --version"
        ).strip()

    def dict(self):
        return {
            'frida_server_name': self.__frida_server_name,
            'frida_server_path': self.__frida_server_full_path,
            'is_running': self.__is_running,
            'version': self.__version
        }

    @__check_is_installed
    def start(self):
        if not self.__is_running:
            subprocess.Popen(
                [adb_path(), '-s', self.__android_device.device_serial, 'shell',
                 'nohup',
                 'su -c', self.__frida_server_full_path, '&']
            )
        self.__is_running = self.__check_if_running()
        return self.__is_running

    @__check_is_installed
    def stop(self):
        if self.is_running:
            self.__android_device.kill_process(
                self.__android_device.get_pid(self.__frida_server_name)
            )
        self.__is_running = self.__check_if_running()
        return not self.__is_running

    def uninstall(self):
        self.__android_device.remove_file(self.__frida_server_full_path)
        return not self.__android_device.file_exists(self.__frida_server_full_path)



import threading

import adbutils

from titandevice.android._device import _AndroidDevice


class AndroidDeviceManager(object):
    __instance: 'AndroidDeviceManager' = None
    __lock = threading.Lock()
    __all_devices: list[_AndroidDevice] = None

    def __new__(cls, *args, **kwargs):
        with cls.__lock:
            if not cls.__instance:
                cls.__instance = super(AndroidDeviceManager, cls).__new__(cls)
        return cls.__instance

    def get_all_devices(self) -> list[_AndroidDevice]:
        if self.__all_devices is None:
            self.__all_devices = [
                _AndroidDevice(device.serial)
                for device in adbutils.adb.device_list()
            ]
            if len(self.__all_devices) == 0:
                from titandevice._titian_exception import AnyDeviceNoFoundException
                raise AnyDeviceNoFoundException()
        return self.__all_devices

    def get_device(self, device_serial: str) -> _AndroidDevice:
        for device in self.get_all_devices():
            if device.device_serial == device_serial:
                return device
        from titandevice._titian_exception import DeviceNoFoundException
        raise DeviceNoFoundException(device_serial)



from titandevice.android._device import _AndroidDevice


class _AndroidFridaManager:
    def __init__(
            self, android_device: _AndroidDevice, frida_server_file_name_prefix: str,
            frida_server_install_path: str
    ):
        self.__android_device = android_device
        self.__frida_server_file_name_prefix = frida_server_file_name_prefix
        if not frida_server_install_path.endswith('/'):
            frida_server_install_path += '/'
        self.__frida_server_install_path = frida_server_install_path

    def get_all_frida(self):
        from titandevice.android._frida import _AndroidFrida
        return [
            _AndroidFrida(self.__android_device, frida_server_path) for
            frida_server_path in
            self.get_all_frida_executable_file_path()
        ]

    def get_all_frida_executable_file_path(self) -> list[str]:
        files = self.__android_device.get_all_files(self.__frida_server_install_path)
        return [self.__get_frida_server_path(file) for file in files if
                file.startswith(self.__frida_server_file_name_prefix)]

    def uninstall_all(self):
        for frida in self.get_all_frida():
            frida.stop()
            frida.uninstall()

    def install_frida(self, input_frida_server_path: str):
        from titandevice.utils import get_file_name_from_path
        file_name = get_file_name_from_path(input_frida_server_path)
        self.__android_device.push(
            input_frida_server_path, self.__frida_server_install_path
        )
        frida_server_file_path = self.__frida_server_install_path + file_name
        self.__android_device.shell(
            'su -c chmod +x ' + frida_server_file_path
        )
        return self.__android_device.file_exists(frida_server_file_path)

    def __get_frida_server_path(self, frida_server_name: str) -> str:
        return f'{self.__frida_server_install_path}{frida_server_name}'

    def stop_all_frida(self):
        for frida in self.get_all_frida():
            frida.stop()
